@@ -32,6 +32,7 @@ def cli():
 @click.option("-n", "--nlo-method", type=str, default="DR", help="tW simluation NLO method", show_default=True)
 @click.option("-x", "--override-selection", type=str, help="override selection with contents of file")
 @click.option("-t", "--use-tptrw", is_flag=True, help="apply top pt reweighting")
+@click.option("-r", "--use-trrw", is_flag=True, help="apply top recursive reweighting")
 @click.option("-i", "--ignore-list", type=str, help="variable ignore list file")
 @click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
 @click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
@@ -50,6 +51,7 @@ def single(
     nlo_method,
     override_selection,
     use_tptrw,
+    use_trrw,
     ignore_list,
     test_size,
     early_stop,
@@ -82,6 +84,7 @@ def single(
         weight_mean=1.0,
         override_selection=override_sel,
         use_tptrw=use_tptrw,
+        use_trrw=use_trrw,
     )
     drop_cols(df, *avoids_for(region))
     if ignore_list:
@@ -118,6 +121,7 @@ def single(
 @click.option("-e", "--early-stop", type=int, default=10, help="number of early stopping rounds", show_default=True)
 @click.option("-x", "--override-selection", type=str, help="override selection with contents of file")
 @click.option("-t", "--use-tptrw", is_flag=True, help="apply top pt reweighting")
+@click.option("-r", "--use-trrw", is_flag=True, help="apply top recursive reweighting")
 @click.option("-i", "--ignore-list", type=str, help="variable ignore list file")
 @click.option("-s", "--test-size", type=float, default=0.40, help="training test size", show_default=True)
 @click.option("-d", "--use-dilep", is_flag=True, help="train with dilepton samples")
@@ -132,12 +136,12 @@ def scan(
     early_stop,
     override_selection,
     use_tptrw,
+    use_trrw,
     ignore_list,
     test_size,
     use_dilep,
     overwrite,
     and_submit,
-    scan_params,
 ):
     """Perform a parameter scan via condor jobs.
 
@@ -196,6 +200,13 @@ def scan(
         pd.get("reg_lambda"),
     )
 
+    log.info("Scan grid:")
+    log.info(" - max_depth: {}".format(pd.get("max_depth")))
+    log.info(" - num_leaves: {}".format(pd.get("num_leaves")))
+    log.info(" - learning_rate: {}".format(pd.get("learning_rate")))
+    log.info(" - min_child_samples: {}".format(pd.get("min_child_samples")))
+    log.info(" - reg_lambda: {}".format(pd.get("reg_lambda")))
+
     for (max_depth, num_leaves, learning_rate, min_child_samples, reg_lambda) in itr:
         suffix = "{}-{}-{}-{}-{}".format(
             max_depth, num_leaves, learning_rate, min_child_samples, reg_lambda,
@@ -211,6 +222,7 @@ def scan(
             "--early-stop {} "
             "{}"
             "{}"
+            "{}"
         ).format(
             datadir,
             region,
@@ -226,6 +238,7 @@ def scan(
             max_depth,
             reg_lambda,
             early_stop,
+            "-r " if use_trrw else "",
             "-t " if use_tptrw else "",
             "-d " if use_dilep else "",
         )
@@ -235,10 +248,10 @@ def scan(
         runs.append(arglist)
         i += 1
 
-    with (ws / "run.sh").open("w") as f:
-        print("#!/bin/bash\n\n", file=f)
+    with (ws / "run.sh").open("w") as outscript:
+        print("#!/bin/bash\n\n", file=outscript)
         for run in runs:
-            print(f"tdub train-single {run}\n", file=f)
+            print(f"tdub train-single {run}\n", file=outscript)
     os.chmod(ws / "run.sh", 0o755)
 
     import pycondor
@@ -531,7 +544,7 @@ def soverb(datadir, selections, use_tptrw):
     """
     from tdub.frames import raw_dataframe, apply_weight_tptrw, satisfying_selection
     from tdub.data import quick_files
-    from tdub.branches import selection_branches
+    from tdub.data import selection_branches
 
     with open(selections, "r") as f:
         selections = json.load(f)
@@ -556,25 +569,66 @@ def soverb(datadir, selections, use_tptrw):
 
 @cli.command("rex-plot", context_settings=dict(max_content_width=92))
 @click.argument("workspace", type=click.Path(exists=True))
-@click.option("-o", "outdir", type=str, default="auto")
-def rex_plot(workspace, outdir):
-    """Generate plots from TRExFitter WORKSPACE using METADATA."""
+@click.option("-o", "--outdir", type=str, default="auto", help="Manual output directory.")
+@click.option("-n", "--n-test", type=int, default=-1, help="Test only n plots (for stacks).")
+@click.option("--stacks/--no-stacks", default=True, help="Do or don't plot stacks.")
+@click.option("--chisq/--no-chisq", default=True, help="Do or don't print chi-square information.")
+@click.option("--impact/--no-impact", default=True, help="Do or don't produce the impact plot.")
+def rex_plot(workspace, outdir, stacks, chisq, impact, n_test):
+    """Generate plots from TRExFitter WORKSPACE."""
     import tdub.rex
     import tdub.config
-
     if outdir == "auto":
         outdir = PosixPath(workspace) / "matplotlib"
     else:
         outdir = PosixPath(outdir)
     outdir.mkdir(exist_ok=True)
-    tdub.config.download_meta_table()
-    tdub.config.set_default_logy()
-    tdub.rex.plot_all_regions(workspace, outdir)
+    if impact:
+        tdub.rex.nuispar_impact_plot_top15(workspace)
+    if stacks:
+        tdub.config.init_meta_table()
+        tdub.config.init_meta_logy()
+        tdub.rex.plot_all_regions(workspace, outdir, stage="pre", show_chisq=chisq, n_test=n_test)
+        tdub.rex.plot_all_regions(workspace, outdir, stage="post", show_chisq=chisq, n_test=n_test)
+    return 0
+
+
+@cli.command("imp-tables", context_settings=dict(max_content_width=92))
+@click.argument("summary-file", type=click.Path(exists=True))
+def imp_tables(summary_file):
+    """Generate importance tables."""
+    import tdub.config
+    import json
+    from textwrap import dedent
+    tdub.config.init_meta_table()
+    summary = json.loads(PosixPath(summary_file).read_text())
+    imp_gain = summary["importances_gain"]
+    imp_split = summary["importances_split"]
+    names = imp_gain.keys()
+    imp_gain = [round(x, 4) for x in imp_gain.values()]
+    imp_split = [round(x, 4) for x in imp_split.values()]
+    imp_gain, names, imp_split = (list(reversed(t)) for t in zip(*sorted(zip(imp_gain, names, imp_split))))
+    print(dedent("""\
+    \\begin{table}[htbp]
+      \\begin{center}
+        \\caption{XXX}
+        \\label{XXX}
+        \\begin{tabular}{lcc}
+          \\toprule
+          Variable & Importance (gain) & Importance (split) \\\\
+          \\midrule"""))
+    for n, g, s in zip(names, imp_gain, imp_split):
+        print("      {} & {} & {} \\\\".format(tdub.config.PLOTTING_META_TABLE["titles"][n]["mpl"], g, s))
+    print(dedent("""\
+          \\bottomrule
+        \\end{tabular}
+      \\end{center}
+    \\end{table}
+    """))
 
 
 def run_cli():
     import tdub.config
-
     tdub.config.AVOID_IN_CLF_1j1b = []
     tdub.config.AVOID_IN_CLF_2j1b = []
     tdub.config.AVOID_IN_CLF_2j2b = []
