@@ -12,15 +12,23 @@ from typing import Union, Set, Dict, Iterable, List, Optional
 
 # external
 import formulate
-import uproot
-from uproot.rootio import ROOTDirectory
+import uproot4 as uproot
+from uproot4.reading import ReadOnlyDirectory
+from uproot4.models.TTree import Model_TTree_v20
 
 # tdub
 import tdub.config
 
 log = logging.getLogger(__name__)
 
-DataSource = Union[ROOTDirectory, str, Iterable[str], os.PathLike, Iterable[os.PathLike]]
+DataSource = Union[
+    str,
+    Iterable[str],
+    os.PathLike,
+    Iterable[os.PathLike],
+    ReadOnlyDirectory,
+    Model_TTree_v20,
+]
 
 
 class Region(Enum):
@@ -43,6 +51,7 @@ class Region(Enum):
     >>> from tdub.data import Region, selection_for
     >>> from tdub.frames import iterative_selection
     >>> df = iterative_selection(files, selection_for(Region.r2j2b))
+
     """
 
     r1j1b = 0
@@ -94,6 +103,7 @@ class Region(Enum):
                 raise ValueError(f"{s} doesn't correspond to a Region")
 
     def __str__(self) -> str:
+        """Convert to string, removing prefix 'r'."""
         return self.name[1:]
 
 
@@ -144,6 +154,7 @@ class SampleInfo:
     MC16d
     >>> sampinfo.tree
     nominal
+
     """
 
     phy_process: str
@@ -153,6 +164,7 @@ class SampleInfo:
     tree: str
 
     def __init__(self, input_file: str) -> None:
+        """Class constructor."""
         if "Data_Data" in input_file:
             self.phy_process = "Data"
             self.dsid = 0
@@ -212,14 +224,14 @@ def avoids_for(region: Union[str, Region]) -> List[str]:
 def branches_from(
     source: DataSource, tree: str = "WtLoop_nominal", ignore_weights: bool = False,
 ) -> List[str]:
-    """Get a list of branches from a `source`.
+    """Get a list of branches from a data source.
 
     If the `source` is a list of files, the first file is the only
     file that is parsed.
 
     Parameters
     ----------
-    source : str, list(str), os.PathLike, list(os.PathLike), or uproot.rootio.ROOTDirectory
+    source : str, list(str), os.PathLike, list(os.PathLike), or uproot File/Tree
         What to parse to get the branch information.
     tree : str
         Name of the tree to get branches from
@@ -231,6 +243,11 @@ def branches_from(
     list(str)
         Branches from the source.
 
+    Raises
+    ------
+    TypeError
+        If `source` can't be used to find a list of branches.
+
     Examples
     --------
     >>> from tdub.data import branches_from
@@ -238,24 +255,25 @@ def branches_from(
     ["pT_lep1", "pT_lep2"]
     >>> branches_from("/path/to/file.root")
     ["pT_lep1", "pT_lep2", "weight_nominal", "weight_tptrw"]
+
     """
-    if isinstance(source, ROOTDirectory):
-        t = source.get(tree)
-    elif isinstance(source, (str, os.PathLike)):
+    if isinstance(source, (str, os.PathLike)):
         t = uproot.open(source).get(tree)
-    else:
+    elif isinstance(source, list):
         t = uproot.open(source[0]).get(tree)
-    branches = [b.decode("utf-8") for b in t.allkeys()]
+    elif isinstance(source, uproot.reading.ReadOnlyDirectory):
+        t = source.get(tree)
+    elif isinstance(source, uproot.behaviors.TTree.TTree):
+        t = source
+    else:
+        raise TypeError("Cannot use source (it is type %s)" % str(type(source)))
+    branches = t.keys()
 
-    # return here if weights are not ignored
-    if not ignore_weights:
-        return branches
+    if ignore_weights:
+        weights = set(filter(re.compile(r"(weight_\w+)").match, branches))
+        branches = set(branches) ^ weights
 
-    # check for weight branches
-    weight_re = re.compile(r"(weight_\w+")
-    weights = set(filter(weight_re.match, branches))
-
-    return list(set(branches) ^ weights)
+    return list(sorted(branches, key=str.lower))
 
 
 def categorize_branches(source: List[str]) -> Dict[str, List[str]]:
@@ -294,6 +312,7 @@ def categorize_branches(source: List[str]) -> Dict[str, List[str]]:
 
     >>> root_file = PosixPath("/path/to/file.root")
     >>> cated = categorize_branches(branches_from(root_file))
+
     """
     metas = {
         "reg1j1b",
@@ -414,6 +433,7 @@ def files_for_tree(
     ['/data/path/ttbar_410472_FS_MC16a_JET_CategoryReduction_JET_JER_EffectiveNP_4__1up.root',
      '/data/path/ttbar_410472_FS_MC16d_JET_CategoryReduction_JET_JER_EffectiveNP_4__1up.root',
      '/data/path/ttbar_410472_FS_MC16e_JET_CategoryReduction_JET_JER_EffectiveNP_4__1up.root']
+
     """
     if campaign is None:
         camp = ""
@@ -602,12 +622,13 @@ def selection_as_numexpr(selection: str) -> str:
     >>> from tdub.data import selection_as_numexpr
     >>> selection_as_numexpr(selection)
     '(reg1j1b == True) & (OS == True) & (mass_lep1jet1 < 155)'
+
     """
     return formulate.from_auto(selection).to_numexpr()
 
 
 def selection_as_root(selection: str) -> str:
-    """Get the ROOT selection string from an arbitrary selection
+    """Get the ROOT selection string from an arbitrary selection.
 
     Parameters
     -----------
@@ -625,6 +646,7 @@ def selection_as_root(selection: str) -> str:
     >>> from tdub.data import selection_as_root
     >>> selection_as_root(selection)
     '(reg1j1b == true) && (OS == true) && (mass_lep1jet1 < 155)'
+
     """
     return formulate.from_auto(selection).to_root()
 
@@ -651,6 +673,7 @@ def selection_branches(selection: str) -> Set[str]:
     >>> selection = "reg2j1b == true && OS == true && (mass_lep1jet1 < 155)"
     >>> minimal_branches(selection)
     {'OS', 'mass_lep1jet1', 'reg2j1b'}
+
     """
     return formulate.from_auto(selection).variables
 
