@@ -65,6 +65,7 @@ class SingleTrainingSummary:
         the binned KS test value for background
     ks_pvalue_bkg : float
         the binned KS test p-value for background
+
     """
 
     def __init__(
@@ -138,6 +139,7 @@ class ResponseHistograms:
         w_test,
         nbins: int = 30,
     ):
+        """Class constructor."""
         self.response_type = response_type
         r_train, r_test = self._eval_model(model, X_train, X_test)
         self._calculate(r_train, r_test, y_train, y_test, w_train, w_test, nbins)
@@ -191,46 +193,22 @@ class ResponseHistograms:
 
     @property
     def ks_sig_test(self) -> float:
-        """Two sample binned KS test for signal.
-
-        Returns
-        -------
-        float
-            The signal KS-test test value.
-        """
+        """float: Two sample binned KS test for signal."""
         return round(float(self.ks_sig[0]), 5)
 
     @property
     def ks_sig_pval(self) -> float:
-        """Two sample binned KS p-value for signal.
-
-        Returns
-        -------
-        float
-            The signal KS-test p-value.
-        """
+        """float: Two sample binned KS p-value for signal."""
         return round(float(self.ks_sig[1]), 5)
 
     @property
     def ks_bkg_test(self) -> float:
-        """Two sample binned KS test for background.
-
-        Returns
-        -------
-        float
-            The background KS-test test value.
-        """
+        """float: Two sample binned KS test for background."""
         return round(float(self.ks_bkg[0]), 5)
 
     @property
     def ks_bkg_pval(self) -> float:
-        """Two sample binned KS p-value for background.
-
-        Returns
-        -------
-        float
-            The background KS-test p-value.
-        """
+        """float: Two sample binned KS p-value for background."""
         return round(float(self.ks_bkg[1]), 5)
 
     def draw(
@@ -308,7 +286,7 @@ class ResponseHistograms:
         ax.legend(handles, labels, loc="upper right", ncol=1, frameon=False, numpoints=1)
         ax.set_ylabel("Arbitrary Units")
         draw_atlas_label(
-            ax, internal_shift=0.2, cme_and_lumi=False, extra_lines=["$tW$ BDT Training"]
+            ax, follow_shift=0.2, cme_and_lumi=False, extra_lines=["$tW$ BDT Training"]
         )
         if xlabel is None:
             if self.response_type == "proba":
@@ -331,6 +309,7 @@ def prepare_from_root(
     scale_sum_weights: bool = True,
     use_campaign_weight: bool = False,
     use_tptrw: bool = False,
+    use_trrw: bool = False,
     test_case_size: Optional[int] = None,
 ) -> Tuple[pd.DataFrame, np.ndarray, np.ndarray]:
     """Prepare the data to train in a region with signal and background ROOT files.
@@ -358,7 +337,9 @@ def prepare_from_root(
     use_campaign_weight : bool
         See the parameter description for :py:func:`tdub.frames.iterative_selection`.
     use_tptrw : bool
-        Apply the top pt reweighing factor.
+        Apply the top pt reweighting factor.
+    use_tptrw : bool
+        Apply the top recursive reweighting factor.
     test_case_size : int, optional
         Prepare a small test case dataset using this many training and
         testing samples.
@@ -421,7 +402,7 @@ def prepare_from_root(
         exclude_avoids=True,
         use_campaign_weight=use_campaign_weight,
         use_tptrw=use_tptrw,
-        entrysteps="1 GB",
+        use_trrw=use_trrw,
     )
 
     if test_case_size is not None:
@@ -493,6 +474,7 @@ def tdub_train_axes(
     -------
     dict(str, Any)
         The argument names and values
+
     """
     return dict(
         learning_rate=learning_rate,
@@ -534,6 +516,7 @@ def sklearn_gen_classifier(
     -------
     sklearn.ensemble.HistGradientBoostingClassifier
         The classifier.
+
     """
     if train_axes is None:
         train_axes = tdub_train_axes()
@@ -606,6 +589,7 @@ def lgbm_gen_classifier(train_axes: Dict[str, Any] = None, **clf_params) -> Base
     -------
     lightgbm.LGBMClassifier
         The classifier.
+
     """
     import lightgbm as lgbm
 
@@ -658,6 +642,7 @@ def lgbm_train_classifier(
     -------
     lightgbm.LGBMClassifier
         The same classifier object passed to the function
+
     """
     X_t, X_v, y_t, y_v, w_t, w_v = train_test_split(
         X_train,
@@ -791,6 +776,8 @@ def single_training(
     fig_roc.subplots_adjust(bottom=0.125, left=0.15)
     fig_roc.savefig("roc.pdf")
 
+    importances_gain = {}
+    importances_split = {}
     # Plot Importances
     if not use_sklearn:
         fig_imp, (ax_imp_gain, ax_imp_split) = plt.subplots(2, 1)
@@ -802,6 +789,16 @@ def single_training(
         ax_imp_split.set_title("")
         fig_imp.subplots_adjust(left=0.475, top=0.975, bottom=0.09, right=0.925)
         fig_imp.savefig("imp.pdf")
+        imp_gain = model.booster_.feature_importance(importance_type="gain")
+        imp_split = model.booster_.feature_importance(importance_type="split")
+        feat_name = model.booster_.feature_name()
+        imp_gain = np.array(imp_gain, dtype=np.float64)
+        imp_split = np.array(imp_split, dtype=np.float64)
+        imp_gain *= (1.0 / np.sum(imp_gain))
+        imp_split *= (1.0 /  np.sum(imp_split))
+        for n, g, s in zip(feat_name, imp_gain, imp_split):
+            importances_gain[n] = float(g)
+            importances_split[n] = float(s)
 
     # Histograms: plot and extract information from them
     proba_histograms = ResponseHistograms(
@@ -841,6 +838,9 @@ def single_training(
     summary["set_params"] = train_axes
     summary["all_params"] = model.get_params()
     summary["best_iteration"] = -1
+    summary["importances_gain"] = importances_gain
+    summary["importances_split"] = importances_split
+
     if early_stopping_rounds is not None:
         if use_sklearn:
             summary["best_iteration"] = int(model.n_iter_)
@@ -869,12 +869,12 @@ def folded_training(
     """Execute a folded training.
 
     Train a :obj:`lightgbm.LGBMClassifier` model using :math:`k`-fold
-    cross validation using the given input data and parameters.  The
+    cross validation using the given input data and parameters. The
     models resulting from the training (and other important training
     information) are saved to ``output_dir``. The entries in the
     ``kfold_kw`` argument are forwarded to the
-    :obj:`sklearn.model_selection.KFold` class for data
-    preprocessing. The default arguments that we use are:
+    :obj:`sklearn.model_selection.KFold` class for data preprocessing.
+    The default arguments that we use are:
 
     - ``n_splits``: 3
     - ``shuffle``: ``True``
@@ -929,6 +929,7 @@ def folded_training(
     ...     "2j2b",
     ...     kfold_kw={"n_splits": 5, "shuffle": True, "random_state": 17}
     ... )
+
     """
     import lightgbm as lgbm
 
@@ -1190,8 +1191,8 @@ def gp_minimize_auc(
     >>> from tdub.train import prepare_from_root, gp_minimize_auc
     >>> gp_minimize_auc("/path/to/data", Region.r2j1b, "DS", "opt_DS_2j1b")
     >>> gp_minimize_auc("/path/to/data", Region.r2j1b, "DR", "opt_DR_2j1b")
-    """
 
+    """
     from skopt.utils import use_named_args
     from skopt.space import Real, Integer
     from skopt.plots import plot_convergence
